#!C:\ci\gdal-split_1657199510871\_h_env\python.exe
# EASY-INSTALL-SCRIPT: 'GDAL==3.0.2','gdal_merge.py'
__requires__ = 'GDAL==3.0.2'
__import__('pkg_resources').run_script('GDAL==3.0.2', 'gdal_merge.py')
